@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1999 Erez Zadok
+ * Copyright (c) 1997-2000 Erez Zadok
  * Copyright (c) 1989 Jan-Simon Pendry
  * Copyright (c) 1989 Imperial College of Science, Technology & Medicine
  * Copyright (c) 1989 The Regents of the University of California.
@@ -38,7 +38,7 @@
  *
  *      %W% (Berkeley) %G%
  *
- * $Id: mapc.c,v 1.1.1.1 1998/11/05 02:04:48 ezk Exp $
+ * $Id: mapc.c,v 1.7 2000/02/07 08:34:50 ezk Exp $
  *
  */
 
@@ -139,6 +139,7 @@ qelem map_list_head = {&map_list_head, &map_list_head};
 static const char *get_full_path(const char *map, const char *path, const char *type);
 static int mapc_meta_search(mnt_map *, char *, char **, int);
 static void mapc_sync(mnt_map *);
+static void mapc_clear(mnt_map *);
 
 /* ROOT MAP */
 static int root_init(mnt_map *, char *, time_t *);
@@ -158,6 +159,7 @@ extern int passwd_search(mnt_map *, char *, char *, char **, time_t *);
 /* HESIOD MAPS */
 #ifdef HAVE_MAP_HESIOD
 extern int amu_hesiod_init(mnt_map *, char *map, time_t *tp);
+extern int hesiod_isup(mnt_map *, char *);
 extern int hesiod_search(mnt_map *, char *, char *, char **, time_t *);
 #endif /* HAVE_MAP_HESIOD */
 
@@ -236,7 +238,7 @@ static map_type maptypes[] =
     "hesiod",
     amu_hesiod_init,
     error_reload,
-    NULL,			/* isup function */
+    hesiod_isup,		/* is Hesiod up or not? */
     hesiod_search,
     error_mtime,
     MAPC_ALL
@@ -487,19 +489,65 @@ mapc_find_wildcard(mnt_map *m)
 
 
 /*
- * Do a map reload
+ * Do a map reload.
+ * Attempt to reload without losing current data by switching the hashes
+ * round.
  */
-static int
+static void
 mapc_reload_map(mnt_map *m)
 {
   int error;
+  kv *maphash[NKVHASH], *tmphash[NKVHASH];
+  time_t t;
+
+  error = (*m->mtime) (m, m->map_name, &t);
+  if (error) {
+    t = m->modify;
+  }
+
+  /*
+   * skip reloading maps that have not been modified, unless
+   * amq -f was used (do_mapc_reload is 0)
+   */
+  if (m->reloads != 0 && do_mapc_reload != 0) {
+    if (t <= m->modify) {
+      plog(XLOG_INFO, "reload of map %s is not needed (in sync)", m->map_name);
+#ifdef DEBUG
+      dlog("map %s last load time is %d, last modify time is %d",
+	   m->map_name, (int) m->modify, (int) t);
+#endif /* DEBUG */
+      return;
+    }
+  }
+
+  /* copy the old hash and zero the map */
+  memcpy((voidp) maphash, (voidp) m->kvhash, sizeof(m->kvhash));
+  memset((voidp) m->kvhash, 0, sizeof(m->kvhash));
 
 #ifdef DEBUG
   dlog("calling map reload on %s", m->map_name);
 #endif /* DEBUG */
   error = (*m->reload) (m, m->map_name, mapc_add_kv);
-  if (error)
-    return error;
+  if (error) {
+    if (m->reloads == 0)
+      plog(XLOG_FATAL, "first time load of map %s failed!", m->map_name);
+    else
+      plog(XLOG_ERROR, "reload of map %s failed - using old values",
+	   m->map_name);
+    mapc_clear(m);
+    memcpy((voidp) m->kvhash, (voidp) maphash, sizeof(m->kvhash));
+  } else {
+    if (m->reloads++ == 0)
+      plog(XLOG_INFO, "first time load of map %s succeeded", m->map_name);
+    else
+      plog(XLOG_INFO, "reload #%d of map %s succeeded",
+	   m->reloads, m->map_name);
+    memcpy((voidp) tmphash, (voidp) m->kvhash, sizeof(m->kvhash));
+    memcpy((voidp) m->kvhash, (voidp) maphash, sizeof(m->kvhash));
+    mapc_clear(m);
+    memcpy((voidp) m->kvhash, (voidp) tmphash, sizeof(m->kvhash));
+    m->modify = t;
+  }
   m->wildcard = 0;
 
 #ifdef DEBUG
@@ -508,8 +556,6 @@ mapc_reload_map(mnt_map *m)
   error = mapc_search(m, wildcard, &m->wildcard);
   if (error)
     m->wildcard = 0;
-
-  return 0;
 }
 
 
@@ -620,6 +666,7 @@ mapc_create(char *map, char *opt, const char *type)
   m->map_name = strdup(map);
   m->refc = 1;
   m->wildcard = 0;
+  m->reloads = 0;
 
   /*
    * synchronize cache with reality
@@ -733,6 +780,7 @@ mapc_meta_search(mnt_map *m, char *key, char **pval, int recurse)
     plog(XLOG_ERROR, "Null map request for %s", key);
     return ENOENT;
   }
+
   if (m->flags & MAPC_SYNC) {
     /*
      * Get modify time...
@@ -740,7 +788,6 @@ mapc_meta_search(mnt_map *m, char *key, char **pval, int recurse)
     time_t t;
     error = (*m->mtime) (m, m->map_name, &t);
     if (error || t > m->modify) {
-      m->modify = t;
       plog(XLOG_INFO, "Map %s is out of date", m->map_name);
       mapc_sync(m);
     }
@@ -880,16 +927,16 @@ mapc_sync(mnt_map *m)
       }
     }
 
-    mapc_clear(m);
-
-    if (m->alloc >= MAPC_ALL)
-      if (mapc_reload_map(m))
-	m->alloc = MAPC_INC;
-    /*
-     * Attempt to find the wildcard entry
-     */
-    if (m->alloc < MAPC_ALL)
+    if (m->alloc >= MAPC_ALL) {
+      /* mapc_reload_map() always works */
+      mapc_reload_map(m);
+    } else {
+      mapc_clear(m);
+      /*
+       * Attempt to find the wildcard entry
+       */
       mapc_find_wildcard(m);
+    }
   }
 }
 
@@ -1051,7 +1098,7 @@ key_already_in_chain(char *keyname, const nfsentry *chain)
 nfsentry *
 make_entry_chain(am_node *mp, const nfsentry *current_chain, int fully_browsable)
 {
-  static u_int last_cookie = ~(u_int) 0 - 1;
+  static u_int last_cookie = (u_int) 2;	/* monotonically increasing */
   static nfsentry chain[MAX_CHAIN];
   static int max_entries = MAX_CHAIN;
   char *key;
@@ -1075,6 +1122,9 @@ make_entry_chain(am_node *mp, const nfsentry *current_chain, int fully_browsable
     return retval;
   }
 
+  if (mp->am_pref)
+    preflen = strlen(mp->am_pref);
+
   /* iterate over keys */
   for (i = 0; i < NKVHASH; i++) {
     kv *k;
@@ -1095,12 +1145,15 @@ make_entry_chain(am_node *mp, const nfsentry *current_chain, int fully_browsable
 
       /*
        * If the map has a prefix-string then check if the key starts with
-       * this * string, and if it does, skip over this prefix.
+       * this string, and if it does, skip over this prefix.  If it has a
+       * prefix and it doesn't match the start of the key, skip it.
        */
-      if (preflen) {
+      if (preflen && (preflen <= (strlen(key)))) {
 	if (!NSTREQ(key, mp->am_pref, preflen))
 	  continue;
 	key += preflen;
+      } else if (preflen) {
+	  continue;
       }
 
       /* no more '/' are allowed, unless browsable_dirs=full was used */
@@ -1123,10 +1176,9 @@ make_entry_chain(am_node *mp, const nfsentry *current_chain, int fully_browsable
       }
 
       /* we have space.  put entry in next cell */
-      --last_cookie;
+      ++last_cookie;
       chain[num_entries].ne_fileid = (u_int) last_cookie;
-      *(u_int *) chain[num_entries].ne_cookie =
-	(u_int) last_cookie;
+      *(u_int *) chain[num_entries].ne_cookie = (u_int) last_cookie;
       chain[num_entries].ne_name = key;
       if (num_entries < max_entries - 1) {	/* link to next one */
 	chain[num_entries].ne_nextentry = &chain[num_entries + 1];
