@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1999 Erez Zadok
+ * Copyright (c) 1997-2000 Erez Zadok
  * Copyright (c) 1990 Jan-Simon Pendry
  * Copyright (c) 1990 Imperial College of Science, Technology & Medicine
  * Copyright (c) 1990 The Regents of the University of California.
@@ -38,7 +38,7 @@
  *
  *      %W% (Berkeley) %G%
  *
- * $Id: srvr_nfs.c,v 1.2 1998/12/27 06:24:48 ezk Exp $
+ * $Id: srvr_nfs.c,v 1.6 2000/01/12 16:44:26 ezk Exp $
  *
  */
 
@@ -136,7 +136,7 @@ start_ping(u_long nfs_version)
     nfs_version = NFS_VERSION;
     plog(XLOG_WARNING, "start_ping: nfs_version = 0 fixed");
   }
-  plog(XLOG_INFO, "start_ping: nfs_version: %d", nfs_version);
+  plog(XLOG_INFO, "start_ping: nfs_version: %d", (int) nfs_version);
 
   rpc_msg_init(&ping_msg, NFS_PROGRAM, nfs_version, NFSPROC_NULL);
 
@@ -187,7 +187,7 @@ got_portmap(voidp pkt, int len, struct sockaddr_in * sa, struct sockaddr_in * ia
 
     if (!error && port) {
 #ifdef DEBUG
-      dlog("got port (%d) for mountd on %s", port, fs->fs_host);
+      dlog("got port (%d) for mountd on %s", (int) port, fs->fs_host);
 #endif /* DEBUG */
       /*
        * Grab the port number.  Portmap sends back
@@ -201,7 +201,7 @@ got_portmap(voidp pkt, int len, struct sockaddr_in * sa, struct sockaddr_in * ia
     } else {
 #ifdef DEBUG
       dlog("Error fetching port for mountd on %s", fs->fs_host);
-      dlog("\t error=%d, port=%d", error, port);
+      dlog("\t error=%d, port=%d", error, (int) port);
 #endif /* DEBUG */
       /*
        * Almost certainly no mountd running on remote host
@@ -282,7 +282,7 @@ recompute_portmap(fserver *fs)
   if (fs->fs_version == 0)
     plog(XLOG_WARNING, "recompute_portmap: nfs_version = 0 fixed");
 
-  plog(XLOG_INFO, "recompute_portmap: NFS version %d", fs->fs_version);
+  plog(XLOG_INFO, "recompute_portmap: NFS version %d", (int) fs->fs_version);
 #ifdef HAVE_FS_NFS3
   if (fs->fs_version == NFS_VERSION3)
     mnt_version = MOUNTVERS3;
@@ -290,7 +290,7 @@ recompute_portmap(fserver *fs)
 #endif /* HAVE_FS_NFS3 */
     mnt_version = MOUNTVERS;
 
-  plog(XLOG_INFO, "Using MOUNT version: %d", mnt_version);
+  plog(XLOG_INFO, "Using MOUNT version: %d", (int) mnt_version);
   call_portmap(fs, nfs_auth, MOUNTPROG, mnt_version, (u_long) IPPROTO_UDP);
 }
 
@@ -667,7 +667,7 @@ find_nfs_srvr(mntfs *mf)
   /* allow overriding if nfsv2 option is specified in mount options */
   if (hasmntopt(&mnt, "nfsv2")) {
     nfs_version = (u_long) 2;	/* nullify any ``vers=X'' statements */
-    nfs_proto = "udp";	/* nullify any ``proto=tcp'' stmts */
+    nfs_proto = "udp";		/* nullify any ``proto=tcp'' statements */
     plog(XLOG_WARNING, "found compatiblity option \"nfsv2\": set options vers=2, proto=udp for host %s", host);
   }
 #endif /* HAVE_NFS_NFSV2_H */
@@ -725,7 +725,7 @@ find_nfs_srvr(mntfs *mf)
       int proto_nfs_version;
       char **p;
 
-      for (p = protocols; *p; p ++) {
+      for (p = protocols; *p; p++) {
 	proto_nfs_version = get_nfs_version(host, ip, nfs_version, *p);
 
 	if (proto_nfs_version > best_nfs_version) {
@@ -756,7 +756,7 @@ find_nfs_srvr(mntfs *mf)
     nfs_proto = "udp";
 
   plog(XLOG_INFO, "Using NFS version %d, protocol %s on host %s",
-       nfs_version, nfs_proto, host);
+       (int) nfs_version, nfs_proto, host);
 
   /*
    * Try to find an existing fs server structure for this host.
