@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1999 Erez Zadok
+ * Copyright (c) 1997-2000 Erez Zadok
  * Copyright (c) 1990 Jan-Simon Pendry
  * Copyright (c) 1990 Imperial College of Science, Technology & Medicine
  * Copyright (c) 1990 The Regents of the University of California.
@@ -38,7 +38,7 @@
  *
  *      %W% (Berkeley) %G%
  *
- * $Id: mount_fs.c,v 1.1.1.1 1998/11/05 02:04:45 ezk Exp $
+ * $Id: mount_fs.c,v 1.10 2000/02/07 10:20:19 ionut Exp $
  *
  */
 
@@ -109,6 +109,7 @@ struct opt_tab mnt_flags[] =
 };
 
 
+/* compute generic mount flags */
 int
 compute_mount_flags(mntent_t *mntp)
 {
@@ -146,6 +147,23 @@ compute_mount_flags(mntent_t *mntp)
 }
 
 
+/* compute generic mount flags for automounter mounts */
+int
+compute_automounter_mount_flags(mntent_t *mntp)
+{
+  int flags = 0;
+
+#ifdef MNT2_GEN_OPT_IGNORE
+  flags |= MNT2_GEN_OPT_IGNORE;
+#endif /* not MNT2_GEN_OPT_IGNORE */
+#ifdef MNT2_GEN_OPT_AUTOMNTFS
+  flags |= MNT2_GEN_OPT_AUTOMNTFS;
+#endif /* not MNT2_GEN_OPT_AUTOMNTFS */
+
+  return flags;
+}
+
+
 int
 mount_fs(mntent_t *mnt, int flags, caddr_t mnt_data, int retry, MTYPE_TYPE type, u_long nfs_version, const char *nfs_proto, const char *mnttabname)
 {
@@ -366,6 +384,9 @@ compute_nfs_args(nfs_args_t *nap, mntent_t *mntp, int genflags, struct sockaddr_
 # ifdef MNT2_NFS_OPT_NFSV3
     nap->flags |= MNT2_NFS_OPT_NFSV3;
 # endif /* MNT2_NFS_OPT_NFSV3 */
+# ifdef MNT2_NFS_OPT_VER3
+    nap->flags |= MNT2_NFS_OPT_VER3;
+# endif /* MNT2_NFS_OPT_VER3 */
   } else
 #endif /* HAVE_FS_NFS3 */
     NFS_FH_DREF(nap->NFS_FH_FIELD, &(fhp->v2.fhs_fh));
@@ -528,12 +549,16 @@ compute_nfs_args(nfs_args_t *nap, mntent_t *mntp, int genflags, struct sockaddr_
      * conf/nfs_prot/nfs_prot_*.h files.
      */
 # ifdef USE_UNCONNECTED_NFS_SOCKETS
-    nap->flags |= MNT2_NFS_OPT_NOCONN;
-    plog(XLOG_WARNING, "noconn option exists, and was turned ON! (May cause NFS hangs on some systems...)");
+    if (!(nap->flags & MNT2_NFS_OPT_NOCONN)) {
+      nap->flags |= MNT2_NFS_OPT_NOCONN;
+      plog(XLOG_WARNING, "noconn option not specified, and was just turned ON (OS override)! (May cause NFS hangs on some systems...)");
+    }
 # endif /* USE_UNCONNECTED_NFS_SOCKETS */
 # ifdef USE_CONNECTED_NFS_SOCKETS
-    nap->flags &= ~MNT2_NFS_OPT_NOCONN;
-    plog(XLOG_WARNING, "noconn option exists, and was turned OFF! (May cause NFS hangs on some systems...)");
+    if (nap->flags & MNT2_NFS_OPT_NOCONN) {
+      nap->flags &= ~MNT2_NFS_OPT_NOCONN;
+      plog(XLOG_WARNING, "noconn option specified, and was just turned OFF (OS override)! (May cause NFS hangs on some systems...)");
+    }
 # endif /* USE_CONNECTED_NFS_SOCKETS */
   }
 #endif /* MNT2_NFS_OPT_NOCONN */
@@ -709,7 +734,8 @@ compute_automounter_nfs_args(nfs_args_t *nap, mntent_t *mntp)
 #endif /* MNT2_NFS_OPT_SYMTTL */
 
   /*
-   * This completes the flags for the HIDE_MOUNT_TYPE code above.
+   * This completes the flags for the HIDE_MOUNT_TYPE  code in the
+   * mount_amfs_toplvl() function in amd/amfs_toplvl.c.
    * Some systems don't have a mount type, but a mount flag.
    */
 #ifdef MNT2_NFS_OPT_AUTO
@@ -718,6 +744,9 @@ compute_automounter_nfs_args(nfs_args_t *nap, mntent_t *mntp)
 #ifdef MNT2_NFS_OPT_IGNORE
   nap->flags |= MNT2_NFS_OPT_IGNORE;
 #endif /* MNT2_NFS_OPT_IGNORE */
+#ifdef MNT2_GEN_OPT_AUTOMNTFS
+  nap->flags |= MNT2_GEN_OPT_AUTOMNTFS;
+#endif /* not MNT2_GEN_OPT_AUTOMNTFS */
 
 #ifdef MNT2_NFS_OPT_DUMBTIMR
   /*
@@ -750,6 +779,13 @@ compute_automounter_nfs_args(nfs_args_t *nap, mntent_t *mntp)
   nap->flags |= MNT2_NFS_OPT_ACDIRMIN | MNT2_NFS_OPT_ACDIRMAX;
 # endif /* defined(MNT2_NFS_OPT_ACDIRMIN) && defined(MNT2_NFS_OPT_ACDIRMAX) */
 #endif /* not MNT2_NFS_OPT_NOAC */
+  /*
+   * Provide a slight bit more security by requiring the kernel to use
+   * reserved ports.
+   */
+#ifdef MNT2_NFS_OPT_RESVPORT
+  nap->flags |= MNT2_NFS_OPT_RESVPORT;
+#endif /* MNT2_NFS_OPT_RESVPORT */
 }
 
 
@@ -812,7 +848,7 @@ print_nfs_args(const nfs_args_t *nap, u_long nfs_version)
   nbp = nap->syncaddr;
   plog(XLOG_DEBUG, "NA->syncaddr {netbuf} 0x%x", (int) nbp);
   kncp = nap->knconf;
-  plog(XLOG_DEBUG, "NA->knconf->semantics %lu", kncp->knc_semantics);
+  plog(XLOG_DEBUG, "NA->knconf->semantics %lu", (unsigned long) kncp->knc_semantics);
   plog(XLOG_DEBUG, "NA->knconf->protofmly \"%s\"", kncp->knc_protofmly);
   plog(XLOG_DEBUG, "NA->knconf->proto \"%s\"", kncp->knc_proto);
   plog(XLOG_DEBUG, "NA->knconf->rdev %lu", kncp->knc_rdev);
@@ -820,9 +856,9 @@ print_nfs_args(const nfs_args_t *nap, u_long nfs_version)
 #else /* not HAVE_TRANSPORT_TYPE_TLI */
   sap = (struct sockaddr_in *) &nap->addr;
   plog(XLOG_DEBUG, "NA->addr {sockaddr_in} (len=%d) = \"%s\"",
-       sizeof(struct sockaddr_in),
+       (int) sizeof(struct sockaddr_in),
        get_hex_string(sizeof(struct sockaddr_in), (const char *)sap));
-#ifdef HAVE_FIELD_STRUCT_SOCKADDR_SA_LEN_off
+#ifdef HAVE_FIELD_STRUCT_SOCKADDR_SA_LEN
   plog(XLOG_DEBUG, "NA->addr.sin_len = \"%d\"", sap->sin_len);
 #endif /* HAVE_FIELD_STRUCT_SOCKADDR_SA_LEN */
   plog(XLOG_DEBUG, "NA->addr.sin_family = \"%d\"", sap->sin_family);
@@ -832,6 +868,10 @@ print_nfs_args(const nfs_args_t *nap, u_long nfs_version)
 #endif /* not HAVE_TRANSPORT_TYPE_TLI */
 
   plog(XLOG_DEBUG, "NA->hostname = \"%s\"", nap->hostname ? nap->hostname : "null");
+#ifdef HAVE_FIELD_NFS_ARGS_T_NAMLEN
+  plog(XLOG_DEBUG, "NA->namlen = %d", nap->namlen);
+#endif /* HAVE_FIELD_NFS_ARGS_T_NAMLEN */
+
 #ifdef MNT2_NFS_OPT_FSNAME
   plog(XLOG_DEBUG, "NA->fsname = \"%s\"", nap->fsname ? nap->fsname : "null");
 #endif /* MNT2_NFS_OPT_FSNAME */
@@ -858,24 +898,27 @@ print_nfs_args(const nfs_args_t *nap, u_long nfs_version)
   plog(XLOG_DEBUG, "NA->sotype = %d", nap->sotype);
 #endif /* HAVE_FIELD_NFS_ARGS_T_SOTYPE */
 #ifdef HAVE_FIELD_NFS_ARGS_T_PROTO
-  plog(XLOG_DEBUG, "NA->proto = %d", nap->proto);
+  plog(XLOG_DEBUG, "NA->proto = %d", (int) nap->proto);
 #endif /* HAVE_FIELD_NFS_ARGS_T_PROTO */
 #ifdef HAVE_FIELD_NFS_ARGS_T_VERSION
   plog(XLOG_DEBUG, "NA->version = %d", nap->version);
 #endif /* HAVE_FIELD_NFS_ARGS_T_VERSION */
 
-  plog(XLOG_DEBUG, "NA->flags = 0x%x", nap->flags);
+  plog(XLOG_DEBUG, "NA->flags = 0x%x", (int) nap->flags);
 
-  plog(XLOG_DEBUG, "NA->rsize = %d", nap->rsize);
-  plog(XLOG_DEBUG, "NA->wsize = %d", nap->wsize);
-  plog(XLOG_DEBUG, "NA->timeo = %d", nap->timeo);
-  plog(XLOG_DEBUG, "NA->retrans = %d", nap->retrans);
+  plog(XLOG_DEBUG, "NA->rsize = %d", (int) nap->rsize);
+  plog(XLOG_DEBUG, "NA->wsize = %d", (int) nap->wsize);
+#ifdef HAVE_FIELD_NFS_ARGS_T_BSIZE
+  plog(XLOG_DEBUG, "NA->bsize = %d", nap->bsize);
+#endif /* HAVE_FIELD_NFS_ARGS_T_BSIZE */
+  plog(XLOG_DEBUG, "NA->timeo = %d", (int) nap->timeo);
+  plog(XLOG_DEBUG, "NA->retrans = %d", (int) nap->retrans);
 
 #ifdef HAVE_FIELD_NFS_ARGS_T_ACREGMIN
-  plog(XLOG_DEBUG, "NA->acregmin = %d", nap->acregmin);
-  plog(XLOG_DEBUG, "NA->acregmax = %d", nap->acregmax);
-  plog(XLOG_DEBUG, "NA->acdirmin = %d", nap->acdirmin);
-  plog(XLOG_DEBUG, "NA->acdirmax = %d", nap->acdirmax);
+  plog(XLOG_DEBUG, "NA->acregmin = %d", (int) nap->acregmin);
+  plog(XLOG_DEBUG, "NA->acregmax = %d", (int) nap->acregmax);
+  plog(XLOG_DEBUG, "NA->acdirmin = %d", (int) nap->acdirmin);
+  plog(XLOG_DEBUG, "NA->acdirmax = %d", (int) nap->acdirmax);
 #endif /* HAVE_FIELD_NFS_ARGS_T_ACREGMIN */
 #ifdef MNTTAB_OPT_SYMTTL
   plog(XLOG_DEBUG, "NA->symttl = %d", nap->symttl);
