@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1999 Erez Zadok
+ * Copyright (c) 1997-2000 Erez Zadok
  * Copyright (c) 1990 Jan-Simon Pendry
  * Copyright (c) 1990 Imperial College of Science, Technology & Medicine
  * Copyright (c) 1990 The Regents of the University of California.
@@ -38,7 +38,7 @@
  *
  *      %W% (Berkeley) %G%
  *
- * $Id: xutil.c,v 1.2 1998/12/27 06:25:24 ezk Exp $
+ * $Id: xutil.c,v 1.10 2000/02/11 02:09:54 ezk Exp $
  *
  */
 
@@ -48,7 +48,12 @@
 #include <am_defs.h>
 #include <amu.h>
 
-FILE *logfp = stderr;		/* Log errors to stderr initially */
+/*
+ * Logfp is the default logging device, and is initialized to stderr by
+ * default in dplog/plog below, and in
+ * amd/amfs_program.c:amfs_program_exec().
+ */
+FILE *logfp = NULL;
 
 static char *am_progname = "unknown";	/* "amd" */
 static char am_hostname[MAXHOSTNAMELEN + 1] = "unknown"; /* Hostname */
@@ -88,6 +93,9 @@ struct opt_tab dbg_opt[] =
   {"daemon", D_DAEMON},		/* Enter daemon mode */
   {"fork", D_FORK},		/* Fork server (nofork = don't fork) */
   {"full", D_FULL},		/* Program trace */
+#ifdef HAVE_CLOCK_GETTIME
+  {"hrtime", D_HRTIME},		/* Print high resolution time stamps */
+#endif /* HAVE_CLOCK_GETTIME */
   /* info service specific debugging (hesiod, nis, etc) */
   {"info", D_INFO},
 # ifdef DEBUG_MEM
@@ -97,6 +105,7 @@ struct opt_tab dbg_opt[] =
   {"str", D_STR},		/* Debug string munging */
   {"test", D_TEST},		/* Full debug - but no daemon */
   {"trace", D_TRACE},		/* Protocol trace */
+  {"xdrtrace", D_XDRTRACE},	/* Trace xdr routines */
   {0, 0}
 };
 #endif /* DEBUG */
@@ -272,30 +281,38 @@ checkup_mem(void)
 
 /*
  * Take a log format string and expand occurrences of %m
- * with the current error code taken from errno.
+ * with the current error code taken from errno.  Make sure
+ * 'e' never gets longer than maxlen characters.
  */
 static void
-expand_error(char *f, char *e)
+expand_error(char *f, char *e, int maxlen)
 {
   extern int sys_nerr;
-  char *p;
+  char *p, *q;
   int error = errno;
+  int len = 0;
 
-  for (p = f; (*e = *p); e++, p++) {
+  for (p = f, q = e; (*q = *p) && len < maxlen; len++, q++, p++) {
     if (p[0] == '%' && p[1] == 'm') {
       const char *errstr;
       if (error < 0 || error >= sys_nerr)
 	errstr = NULL;
       else
-	errstr = sys_errlist[error];
+#ifdef HAVE_STRERROR
+	errstr = strerror(error);
+#else /* not HAVE_STRERROR */
+        errstr = sys_errlist[error];
+#endif /* not HAVE_STRERROR */
       if (errstr)
-	strcpy(e, errstr);
+	strcpy(q, errstr);
       else
-	sprintf(e, "Error %d", error);
-      e += strlen(e) - 1;
+	sprintf(q, "Error %d", error);
+      len += strlen(q) - 1;
+      q += strlen(q) - 1;
       p++;
     }
   }
+  e[maxlen-1] = '\0';		/* null terminate, to be sure */
 }
 
 
@@ -307,13 +324,34 @@ show_time_host_and_name(int lvl)
 {
   static time_t last_t = 0;
   static char *last_ctime = 0;
-  time_t t = clocktime();
+  time_t t;
+#ifdef HAVE_CLOCK_GETTIME
+  struct timespec ts;
+#endif /* HAVE_CLOCK_GETTIME */
+  char nsecs[11] = "";	/* '.' + 9 digits + '\0' */
   char *sev;
 
+#ifdef HAVE_CLOCK_GETTIME
+  /*
+   * Some systems (AIX 4.3) seem to implement clock_gettime() as stub
+   * returning ENOSYS.
+   */
+  if (clock_gettime(CLOCK_REALTIME, &ts) == 0) {
+    t = ts.tv_sec;
+#ifdef DEBUG
+    amuDebug(D_HRTIME)
+      sprintf(nsecs, ".%09ld", ts.tv_nsec);
+#endif /* DEBUG */
+  }
+  else
+#endif /* HAVE_CLOCK_GETTIME */
+    t = clocktime();
+
   if (t != last_t) {
     last_ctime = ctime(&t);
     last_t = t;
   }
+
   switch (lvl) {
   case XLOG_FATAL:
     sev = "fatal:";
@@ -343,8 +381,8 @@ show_time_host_and_name(int lvl)
     sev = "hmm:  ";
     break;
   }
-  fprintf(logfp, "%15.15s %s %s[%ld]/%s ",
-	  last_ctime + 4, am_get_hostname(),
+  fprintf(logfp, "%15.15s%s %s %s[%ld]/%s ",
+	  last_ctime + 4, nsecs, am_get_hostname(),
 	  am_get_progname(),
 	  (long) am_mypid,
 	  sev);
@@ -367,6 +405,9 @@ dplog(char *fmt, ...)
 {
   va_list ap;
 
+  if (!logfp)
+    logfp = stderr;		/* initialize before possible first use */
+
   va_start(ap, fmt);
   real_plog(XLOG_DEBUG, fmt, ap);
   va_end(ap);
@@ -379,6 +420,9 @@ plog(int lvl, char *fmt, ...)
 {
   va_list ap;
 
+  if (!logfp)
+    logfp = stderr;		/* initialize before possible first use */
+
   va_start(ap, fmt);
   real_plog(lvl, fmt, ap);
   va_end(ap);
@@ -401,9 +445,19 @@ real_plog(int lvl, char *fmt, va_list vargs)
   checkup_mem();
 #endif /* DEBUG_MEM */
 
-  expand_error(fmt, efmt);
+  expand_error(fmt, efmt, 1024);
 
+#ifdef HAVE_VSNPRINTF
+  vsnprintf(ptr, 1024, efmt, vargs);
+#else /* not HAVE_VSNPRINTF */
+  /*
+   * XXX: ptr is 1024 bytes long.  It is possible to write into it
+   * more than 1024 bytes, if efmt is already large, and vargs expand
+   * as well.  This is not as safe as using vsnprintf().
+   */
   vsprintf(ptr, efmt, vargs);
+  msg[1023] = '\0';		/* null terminate, to be sure */
+#endif /* not HAVE_VSNPRINTF */
 
   ptr += strlen(ptr);
   if (ptr[-1] == '\n')
@@ -774,6 +828,7 @@ switch_to_logfile(char *logfile, int old_umask)
     (void) fclose(logfp);
   logfp = new_logfp;
 
+  plog(XLOG_INFO, "switched to logfile \"%s\"", logfile);
   return 0;
 }
 
